@@ -1,17 +1,21 @@
 # utils -----
-css_px <- function(x, format = "%.0fpx"){
-  ifelse( is.na(x), "inherit",
-          ifelse( x < 0.001, "0", sprintf(format, x)) )
+css_pt <- function(x, digits = 0){
+  x <- ifelse( is.na(x), "inherit",
+          ifelse( x < 0.001, "0",
+                  paste0(format_double(x, digits = digits),"pt")))
+  x
 }
-
-css_pt <- function(x){
-  ifelse( is.na(x), "inherit",
-          ifelse( x < 0.001, "0", sprintf("%.0fpt", x)) )
+css_no_unit <- function(x, digits = 0){
+  x <- ifelse( is.na(x), "inherit",
+          ifelse( x < 0.001, "0",
+                  format_double(x, digits = digits)))
+  x
 }
 
 border_css <- function(color, width, style, side){
   style[!style %in% c("dotted", "dashed", "solid")] <- "solid"
-  sprintf("border-%s: %s %s %s;", side, css_px(width, "%.2fpx"), style, colcodecss(color))
+  x <- sprintf("border-%s: %s %s %s;", side, css_pt(width, 2), style, colcodecss(color))
+  x
 }
 border_wml <- function(color, width, style, side){
   width[style %in% c("none")] <- 0
@@ -50,7 +54,7 @@ border_pml <- function(color, width, style, side){
 format.complex_tabpart <- function( x, type = "wml", header = FALSE,
                                     split = FALSE, colwidth = TRUE, ... ){
   stopifnot(length(type) == 1)
-  stopifnot( type %in% c("wml", "pml", "html") )
+  stopifnot( type %in% c("wml", "pml") )
 
   if(!colwidth){
     x$colwidths[] <- NA_real_
@@ -101,13 +105,11 @@ format.complex_tabpart <- function( x, type = "wml", header = FALSE,
                      span_rows = x$spans$rows,
                      span_columns = x$spans$columns, x$colwidths, x$rowheights, x$hrule, text.align=x$styles$pars$text.align$data)
   setDT(cells)
-  cells <- dcast(cells, row_id ~ col_id, drop=FALSE, fill="", value.var = "cell_str", fun.aggregate = I)
-  cells$row_id <- NULL
+  cells <- dcast(cells, ft_row_id ~ col_id, drop=FALSE, fill="", value.var = "cell_str", fun.aggregate = I)
+  cells$ft_row_id <- NULL
   cells <- apply(as.matrix(cells), 1, paste0, collapse = "")
 
-  if( type == "html"){
-    rows <- paste0(sprintf("<tr%s>", ifelse(x$hrule %in% "exact", "", " style=\"overflow-wrap:break-word;\"")), cells, "</tr>")
-  } else if( type == "wml"){
+  if( type == "wml"){
     rows <- paste0( "<w:tr><w:trPr>",
             ifelse(split, "", "<w:cantSplit/>"),
             "<w:trHeight w:val=",
